/*
 * core/Distributor.cpp  --  Part of the CinePaint plug-in "Bracketing_to_HDR"
 *
 * Copyright (c) 2005-2006  Hartmut Sbosny  <hartmut.sbosny@gmx.de>
 *
 * LICENSE:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/**
  @file Distributor.cpp
   
  Utility for communication between several classes. See "Distributor.hpp"
*/

#include <cstdio>
#include "Distributor.hpp"


//#define BR_DEBUG_DISTRIBUTOR

//==========================
//  DistributorBase...
//==========================

/**
*   login()  -  login a participant. 
* 
*   @param fp: pointer of the callback function, which shall be called.
*   @param userdata: data, which shall be given as second argument in
*      the callback function.
* 
*   Notes: 
*    - Double logins of identic (fp,userdata)-pairs are not prevented; would 
*       lead to (ordinar undangerous) multiple calls.
*    - Several participants can have identic fp's if static class element
*       functions are used (all instances have here the same fp!)
*    - Several participants can have identic userdata if not the this-pointer
*       is used, but something others.
*   
*/
void DistributorBase::login (Callback* fp, void* userdata)
{
    array_.push_back (Entry(fp,userdata));

#ifdef BR_DEBUG_DISTRIBUTOR    
    printf("DistributorBase::%s( fp=%p, userdata=%p ): new size()=%d\n", 
           __func__, fp, userdata, array_.size());
    //report(0);
#endif
}


/**
*   Logout a participant identified by its callback pointer `fp'.
* 
*   @param fp: function pointer as it was logged in (should be unique in array)
* 
*   Note: Call with non-registered or already removed Callbacks harmless
*/  
void DistributorBase::logout (Callback* fp)
{
    for (unsigned i=0; i < array_.size(); i++) {
      if (array_[i].fp == fp) {
        array_.erase (array_.begin()+i);

#ifdef BR_DEBUG_DISTRIBUTOR        
        printf("DistributorBase::%s( fp=%p ): new size()=%d\n",
               __func__, (void*)fp, array_.size());
#endif 
        return;
      }
    }
    printf("DistributorBase::%s(callback=%p: not found): size()=%d\n",
           __func__, (void*)fp, array_.size());
}


/**
*   Logout a participant identified by its userdata.
* 
*   @param userdata: userdata as it was logged in (should be unique in array)
* 
*   Note: Call with non-registered or already removed Users harmless
*/  
void DistributorBase::logout (void* userdata)
{
    for (unsigned i=0; i < array_.size(); i++) {  
      if (array_[i].userdata == userdata) {
        array_.erase (array_.begin()+i);

#ifdef BR_DEBUG_DISTRIBUTOR        
        printf("DistributorBase::%s( userdata=%p ): new size()=%d\n",
               __func__, userdata, array_.size());
#endif 
        return;
      }
    }
    printf("DistributorBase::%s( userdata=%p: not found ): size()=%d\n", 
           __func__, userdata, array_.size());
}


/**
*   distribute() -  Calls the callback functions `fp(pData,userdata)' for all
*    logged in participants with `pData' and the deposited `userdata' as second
*    argument.
* 
*   @param pData: void pointer to the data ("info"), which is to distribute
*/

void DistributorBase::distribute (const void* pData)
{
    for (size_t i=0; i < array_.size(); i++)
      array_[i].fp (pData, array_[i].userdata);
}


/**
*   report() -  List current array of callbacks and userdata on console.
* 
*   @param label: Prefix of the report title, e.g. "Event"
*/  
void DistributorBase::report (const char* label) const
{
    printf("\"%s\"-Distributor-Report:\n", label);
    for (unsigned i=0; i < array_.size(); i++)
      printf("\t%d: callback = %p,  userdata = %p\n", i, (void*)array_[i].fp, array_[i].userdata);
}


// END OF FILE
