/*
 * core/mergeHdr_PackScheme2D_RGB.hpp  --  Part of the CinePaint plug-in "Bracketing_to_HDR"
 *
 * Copyright (c) 2005-2006  Hartmut Sbosny  <hartmut.sbosny@gmx.de>
 *
 * LICENSE:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/**
  @file mergeHdr_PackScheme2D_RGB.hpp
  
  Merging of a typed Pack of RGB_U8 image buffers into one single HDR(float)
   Rgb-image using three given response curves, a given vector of exposure
   times and a given weight function.
   
  Contents: 
   - merge_Hdr_RGB_U8()
   - merge_LogHdr_RGB_U8()
   
  Seit der Umstellung von `WeightTable auf `WeightFunc, wo die Tabelle nicht
   mehr gerantiert ist, sondern erzeugt werden muss, kann `weight nicht mehr
   als Konstantreferenz uebergeben werden, falls Tabelle innerhalb der Funktion
   erzeugt werden koennen soll. Vorher erzeugen? 
*/
#ifndef mergeHdr_PackScheme2D_RGB_hpp
#define mergeHdr_PackScheme2D_RGB_hpp


#include "TNT/tnt_array1d.hpp"
#include "br_PackImgScheme2D.hpp"     // PackImgScheme2D_RGB_U*
#include "br_Image.hpp"               // Image
#include "WeightFunc.hpp"             // WeightFunc_U8*
#include "ResponseFunc_U16.hpp"       // ResponseFunc_U16
#include "ProgressInfo.hpp"


namespace br {


/**+*************************************************************************\n
  merge_Hdr_RGB_U8()
 
  @param pack: Typed Pack of RGB_U8 image buffers.

  @param logX_R|G|B: Logarithm. inverse of the response function z=f(X) - 
     logX_R[z]: Logarithm exposure `log(X)' to pixel value `z' in channel
     R accordingly to z = f(X) = f(E*t);  f^-1(z) = X = E*t;  g(z) = log(X) 
     = log(E)+log(t)  with g(z):=log[f^-1(z)].
 
  @param weight: Weight function object. Ordinarily jene auch bei der 
     logX_*-Berechng verwendete. Weight[z]: weight of value z.

  @param progressinfo: Pointer to a ProgressInfo instance (default 0).
 
  @param protocol_to_file,protocol_to_stdout: A bad-pixel-protocol to a file
     (default filename) and/or to stdout (y/n).
  
  @return: HDR<float>-RGB-Bild: die aus log(E)=log(X)-log(t) resultierenden
     E-Daten: logE(z) = logX(z) - logTimes, oder vielmehr, exp(logE).
 
  @NOTE: Wenn ein Pixel in *allen* Bildern auf einem Randwert (0 oder zmax),
   dann wegen Wichtung sum_w=0 und sum_e=0 und wir bekaemen "0/0". Tatsaechlich
   wollen wir dann minimalen HDR-Wert (E) fuer z=0 und maximalen fuer z=zmax.
   Der kleinste reale E-Wert (den diese Belichtungsreihe erfassen koennte) ist
   der zu z=0 im Bild mit der *laengsten* Belichtungszeit, der groesste noch
   erfassbare der zu z=z_max im Bild mit der *kuerzesten* Bel-Zeit. Gemaess 
   logE = logX - logtimes und dem Umstand, dass die Bilder in aufsteigender
   Belicht.-Zeit geordnet, also
            logE_min = logX[0] - logtimes[nlayers-1]
            logE_max = logX[z_max] - logtimes[0]

******************************************************************************/
br::Image    // IMAGE_RGB_F32
merge_Hdr_RGB_U8 (      const br::PackImgScheme2D_RGB_U8 & pack,
                        const TNT::Array1D<double> & logX_R,
                        const TNT::Array1D<double> & logX_G,
                        const TNT::Array1D<double> & logX_B,
                        const WeightFunc_U8        & weight,
                        bool                       mark_bad_pixel     = false,
                        ProgressInfo*              progressinfo       = 0,
                        bool                       protocol_to_file   = false, 
                        bool                       protocol_to_stdout = false );


/**+*************************************************************************\n
  merge_LogHdr_RGB_U8()  
  
  Logarithm. HDR-Bild extra zu bauen deshalb interessant, weil beim gewoehnl.
   fuer alle Werte ein `exp(z) auftritt, das hier gespart werden kann. Bei 
   N Pixeln und P LDR-Bildern sind das 3*N*P exp()-Operationen.
  
  @param *: siehe `merge_Hdr_...()'
  @return: logarithmiertes HDR<float>-Bild: die gemaess log(E)=log(X)-log(t) 
    resultierenden log(E)-Daten.
******************************************************************************/
br::Image    // IMAGE_RGB_F32
merge_LogHdr_RGB_U8 (   const br::PackImgScheme2D_RGB_U8 & pack,
                        const TNT::Array1D<double> & logX_R,
                        const TNT::Array1D<double> & logX_G,
                        const TNT::Array1D<double> & logX_B,
                        const WeightFunc_U8        & weight,
                        bool                       mark_bad_pixel     = false,
                        ProgressInfo*              progressinfo       = 0,
                        bool                       protocol_to_file   = false, 
                        bool                       protocol_to_stdout = false );


/**+*************************************************************************\n
  merge_Hdr_RGB_U16()  --  for ResponseFunc_U16 arguments.
                            See: merge_Hdr_RGB_U8()
******************************************************************************/
br::Image    // IMAGE_RGB_F32
merge_Hdr_RGB_U16 (     const br::PackImgScheme2D_RGB_U16 & pack,
                        const ResponseFunc_U16 & logX_R,
                        const ResponseFunc_U16 & logX_G,
                        const ResponseFunc_U16 & logX_B,
                        const WeightFunc_U16   & weight,
                        bool                     mark_bad_pixel     = false,
                        ProgressInfo*            progressinfo       = 0,
                        bool                     protocol_to_file   = false, 
                        bool                     protocol_to_stdout = false );

/**+*************************************************************************\n
  Provisorium, um ins momentane HdrManager-Regime zu passen: Wir interpretieren
   die logX-Arrays als Gitterwerte einer tabellierten Responsefunktion zu den
   Stuetzstellen z=256*i+127. Eigentlich muessten diese Responsefunktionen als 
   Argument uebergeben werden, was erlaubte, die Funktionen ausserhalb zu init.
   (spline oder tabellieren) und so mehrfach zu verwenden.
******************************************************************************/
br::Image    // IMAGE_RGB_F32
merge_Hdr_RGB_U16 (     const br::PackImgScheme2D_RGB_U16 & pack,
                        const TNT::Array1D<double> & logX_R,
                        const TNT::Array1D<double> & logX_G,
                        const TNT::Array1D<double> & logX_B,
                        const WeightFunc_U16       & weight,
                        bool                       mark_bad_pixel     = false,
                        ProgressInfo*              progressinfo       = 0,
                        bool                       protocol_to_file   = false, 
                        bool                       protocol_to_stdout = false );

/**+*************************************************************************\n
  merge_LogHdr_RGB_U16()  --  for ResponseFunc_U16 arguments
******************************************************************************/
br::Image    // IMAGE_RGB_F32
merge_LogHdr_RGB_U16 (  const br::PackImgScheme2D_RGB_U16 & pack,
                        const ResponseFunc_U16 & logX_R,
                        const ResponseFunc_U16 & logX_G,
                        const ResponseFunc_U16 & logX_B,
                        const WeightFunc_U16   & weight,
                        bool                     mark_bad_pixel     = false,
                        ProgressInfo*            progressinfo       = 0,
                        bool                     protocol_to_file   = false, 
                        bool                     protocol_to_stdout = false );

/**+*************************************************************************\n
  merge_LogHdr_RGB_U16()  --  for Array1D arguments
   Provisorium... (see: merge_Hdr_RGB_U16(Array1D) ).
******************************************************************************/
br::Image    // IMAGE_RGB_F32
merge_LogHdr_RGB_U16 (  const br::PackImgScheme2D_RGB_U16 & pack,
                        const TNT::Array1D<double> & logX_R,
                        const TNT::Array1D<double> & logX_G,
                        const TNT::Array1D<double> & logX_B,
                        const WeightFunc_U16       & weight,
                        bool                       mark_bad_pixel     = false,
                        ProgressInfo*              progressinfo       = 0,
                        bool                       protocol_to_file   = false, 
                        bool                       protocol_to_stdout = false );

}  // namespace "br"

#endif // mergeHdr_PackScheme2D_RGB_hpp

// END OF FILE
